/*
This is part of the audio CD player library
Copyright (C)1998-2000 Tony Arcieri

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

#ifndef _CDAUDIO_H
#define _CDAUDIO_H

/* Size and information definitions */
#define LIBCDAUDIO_VERSION_MAJOR 1
#define LIBCDAUDIO_VERSION_MINOR 0
#define LIBCDAUDIO_VERSION_MICRO 0

#define LIBCDAUDIO_VERSION \
	((LIBCDAUDIO_VERSION_MAJOR << 16) | \
	 (LIBCDAUDIO_VERSION_MINOR <<  8) | \
	 (LIBCDAUDIO_VERSION_MICRO))

#define MAX_TRACKS		100
#define CDINDEX_ID_SIZE		30

enum { CDAUDIO_TRACK_AUDIO, CDAUDIO_TRACK_DATA };
enum { CD_ACCESS_LOCAL, CD_ACCESS_REMOTE };

#define PLAY_START_TRACK	0
#define PLAY_END_TRACK		1
#define PLAY_START_POSITION	2
#define PLAY_END_POSITION	4

/* Backwards compatibility definitions */
#define cddb_server		cd_server
#define cddb_host		cd_host
#define CDDB_MODE_CDDBP		CDDB_CDDBP
#define CDDB_MODE_HTTP		CDDB_HTTP
#define CDINDEX_MODE_HTTP	CDINDEX_HTTP
#define COVERART_MODE_HTTP	COVERART_HTTP
#define CDDB_ACCESS_LOCAL	CD_ACCESS_LOCAL
#define CDDB_ACCESS_REMOTE	CD_ACCESS_REMOTE

#define HTTP_DEFAULT_PORT	80
#define CDDBP_DEFAULT_PORT	888

#define CDDB_PROTOCOL_LEVEL	3

typedef int cd_desc_t;
typedef enum { CDAUDIO_PLAYING, CDAUDIO_PAUSED, CDAUDIO_COMPLETED, CDAUDIO_NOSTATUS, CDAUDIO_INVALID, CDAUDIO_ERROR } cd_status_data;
typedef enum { GENRE_UNKNOWN, GENRE_BLUES, GENRE_CLASSICAL, GENRE_COUNTRY, GENRE_DATA, GENRE_FOLK, GENRE_JAZZ, GENRE_MISC, GENRE_NEWAGE, GENRE_REGGAE, GENRE_ROCK, GENRE_SOUNDTRACK, GENRE_CDINDEX } genre_data;
typedef enum { CONN_UNKNOWN, CDDB_CDDBP, CDDB_HTTP, CDINDEX_HTTP, COVERART_HTTP } cd_connection_type;

/* MSF data structure */
struct disc_timeval {
   int minutes;
   int seconds;
   int frames;
};

/* Disc status structure */
struct disc_status {
   int status_present;                          /* Is disc present? */
   int status_mode;                             /* Disc mode */
   struct disc_timeval status_disc_time;        /* Current disc time */
   struct disc_timeval status_track_time;       /* Current track time */
   int status_current_track;                    /* Current track */
};

/* Track specific information */
struct track_info {
   struct disc_timeval track_length;            /* Length of track */
   struct disc_timeval track_pos;               /* Position of track */
   int track_lba;                               /* Logical Block Address */
   int track_type;                              /* CDAUDIO_TRACK_AUDIO or CDAUDIO_TRACK_DATA */
};

/* Disc information such as current track, amount played, etc */
struct disc_info {
   int disc_present;                            /* Is disc present? */
   int disc_mode;                               /* Current disc mode */
   struct disc_timeval disc_track_time;         /* Current track time */
   struct disc_timeval disc_time;               /* Current disc time */
   struct disc_timeval disc_length;             /* Total disc length */
   int disc_current_track;                      /* Current track */
   int disc_first_track;                        /* First track on the disc */
   int disc_total_tracks;                       /* Number of tracks on disc */
   struct track_info disc_track[MAX_TRACKS];    /* Track specific information */
};

/* Invisible volume structure */
struct __volume {
   int left;
   int right;
};

/* Volume structure */
struct disc_volume {
   struct __volume vol_front;                   /* Normal volume settings */
   struct __volume vol_back;                    /* Surround sound volume settings */
};

/* Server connection information */
struct cd_server {
   char *server_name;
   int server_port;
};

/* CDDB or CD Index host information */
struct cd_host {
   struct cd_server host_server;
   char *host_addressing;
   cd_connection_type host_protocol;
};

/* Server connection structure */
typedef struct cd_conn {
   int sock;
   struct cd_host host;
   struct cd_server *proxy;
   char *http_string;
   char *auth_string;
} *cd_conn_t;

/* Query data structure */
struct cd_query_data {
   int query_tracks;
   int query_offset[MAX_TRACKS];
   int query_length;
};

enum { QUERY_NOMATCH, QUERY_EXACT, QUERY_INEXACT };

/* CDDB query structures */
struct query_list_entry {
   genre_data list_genre;
   unsigned long list_id;
   char list_cdindex_id[CDINDEX_ID_SIZE];
   char *list_artist;
   char *list_title;
};

struct cd_query {
   int query_match;
   int query_matches;
   struct query_list_entry **query_list;
};

struct art_query_entry {
   char *list_url;
   char *list_artist;
   char *list_title;
};

struct art_query {
   int query_matches;
   struct art_query_entry **query_list;
};

struct art_data {
   int art_present;
   int art_length;
   char *art_data;
};

/* Track data structure */
struct track_data {
   char *track_name;
   char *track_artist;
   char *track_extended;
};

enum { CDAUDIO_SINGLE_ARTIST, CDAUDIO_MULTIPLE_ARTIST };

/* Disc data structure */
struct disc_data {
   int data_active;
   int data_total_tracks;
   int data_artist_type;
   genre_data data_genre;
   int data_revision;
   unsigned long data_cddb_id;
   char data_cdindex_id[CDINDEX_ID_SIZE];
   char *data_name;
   char *data_artist;
   char *data_extended;
   struct track_data **data_track;
   struct cd_query_data data_query;
};

/* CDDB/CD Index/Cover Art server list */
struct cd_server_list {
   int list_length;
   char **list_server;
};

/* libcdaudio configuration data */
struct cd_conf {
   int conf_access;
   char *conf_proxy;
   char *conf_email;
   struct cd_server_list conf_list;
};

/* CD Audio function prototypes */
long cdaudio_getversion();
cd_desc_t cd_init_device(const char *device_name);
int cd_close(cd_desc_t cd_desc);
int cd_stat(cd_desc_t cd_desc, struct disc_info *disc);
int cd_poll(cd_desc_t cd_desc, struct disc_status *status);
int cd_play_frames(cd_desc_t cd_desc, int startframe, int endframe);
int cd_play_track_pos(cd_desc_t cd_desc, int starttrack, int endtrack, int startpos);
int cd_play_track(cd_desc_t cd_desc, int starttrack, int endtrack);
int cd_play_pos(cd_desc_t cd_desc, int track, int startpos);
int cd_play(cd_desc_t cd_desc, int track);
int cd_stop(cd_desc_t cd_desc);
int cd_pause(cd_desc_t cd_desc);
int cd_resume(cd_desc_t cd_desc);
int cd_tray_eject(cd_desc_t cd_desc);
int cd_tray_close(cd_desc_t cd_desc);
int cd_get_volume(cd_desc_t cd_desc, struct disc_volume *vol);
int cd_set_volume(cd_desc_t cd_desc, struct disc_volume vol);
int cd_frames_to_lba(int frames);
int cd_lba_to_frames(int lba);
int cd_msf_to_frames(struct disc_timeval time);
int cd_msf_to_lba(struct disc_timeval time);
void cd_frames_to_msf(struct disc_timeval *time, int frames);
void cd_lba_to_msf(struct disc_timeval *time, int lba);
int cd_track_advance(cd_desc_t cd_desc, int endtrack, struct disc_timeval time);
int cd_advance(cd_desc_t cd_desc, struct disc_timeval time);
int cd_update(struct disc_info *disc, struct disc_status status);
int cd_playctl(cd_desc_t cd_desc, int options, int start_track, ...);
int cd_get_query_data(cd_desc_t cd_desc, struct cd_query_data *query);

/* Network operations function prototypes */
int cd_process_url(struct cd_host *host, const char *source_url);
void cd_server_free(struct cd_server *server);
void cd_host_free(struct cd_host *host);
int cd_connect(const struct cd_server *server);
cd_conn_t cd_conn_connect(const char *url);
int cd_conn_query(cd_desc_t cd_desc, cd_conn_t conn, struct cd_query *query);
int cd_conn_read(cd_conn_t conn, struct disc_data *data, struct query_list_entry *entry);
void cd_conn_close(cd_conn_t conn);

/* Data managment function prototypes */
void data_set_disc_title_parsing(char *token);
void data_set_track_parsing_token(char *token);
void data_init(struct disc_data *data);
int data_alloc(struct disc_data *data, int tracks);
void data_free(struct disc_data *data);

#endif  /* _CDAUDIO_H */
